function barcode
% line 3 until line 82 are GUI questions about how to format figures
try
    data = csvread('SCREENSTATE_DATA.csv');
    data(:, 6) = mod(data(:,5),1);
    
    fileID = fopen('SCREENSTATE_DEVICES.txt');
    ud = textscan(fileID, '%s');
    
    for n = 1 : length(ud{1,1})
        ud1 = ud{1,1}{n,1};
        unique_devices{n} = ud1;
    end
    
catch
    errordlg({'Could not find data'...
        ''...
        'Please run CSV2DATA, and ensure that you have:'...
        '     SCREENSTATE_DATA.csv & '...
        '     SCREENSTATE_DEVICES.txt'...
        'in your current directory.'})
end

clear fileID ud ud1 n

ok = 0;
while ok == 0
    [pnum,ok] = listdlg('PromptString','which participants do you want?',...
        'SelectionMode','multiple',...
        'ListSize',[250 250],...
        'ListString',unique_devices);
end

ok = 0;
while ok == 0
    [type,ok] = listdlg('PromptString','how many days do you want?',...
        'SelectionMode','single',...
        'ListSize',[250 50],...
        'ListString',{'all days', '1 day heatmap', '7 day heatmap', 'weekend vs. weekday heatmap'});
end
if type == 1
    alldays = questdlg('Do you want all days for each figure?',...
        'all days?', ...
        'yes', 'no', 'yes');
    
    if strcmp('no', alldays) == 1
        axisy = inputdlg({'how many days do you want for each figure?'},...
            'hist params', 1, {'15'});
        axisy = str2double(axisy{1});
    end
end

addtitle = questdlg('Do you want a title for each figure?',...
    'add title?', ...
    'yes', 'no', 'yes');

switch addtitle
    case 'yes'
        addtitle = 1;
    case 'no'
        addtitle = 0;
end

savefigure = questdlg('Do you want to save each figure?',...
    'save figure?', ...
    'yes', 'no', 'yes');
switch savefigure
    case 'yes'
        savefigure = 1;
    case 'no'
        savefigure = 0;
end

if     type == 1
    ndays = 0;
elseif type == 2
    ndays = 1;
elseif type == 3
    ndays = 7;
elseif type == 4
    ndays = 2;
end

% creates figure for the each participants required
for eachID = 1:length(pnum)
    
    fig(eachID) = figure; hold on
    screendata  = data;
    UserID      = screendata(:, 2)==pnum(eachID);
    screendata  = screendata(UserID, :);
    
    % create  default variables (x refers to first phone use)
    day      = 1;
    midnight = 1;
    x        = 2;
    
    % assigns the day of the first Saturday and Sunday to variables
    firstsat = 7-(mod(floor(screendata(1, 5)),7));
    firstsun = firstsat+1;
    weekday  = 7-firstsun;
    
    % make the current time a number between 0 and 1 (midday is .5)
    time = screendata(:,5)-(floor(screendata(1, 5)));
    
    % draw all phone uses for this participant
    while true
        while time(x-1) < day
            
            if length(time) < x
                break
            end
            
            begin     = (time(x-1) - (day-1));
            duration  = (time(x) - time(x-1));
            
            % add red lines indicating where Saturdays begin
            if ndays == 0 && mod((day-firstsat),7)==0
                h           = line([0 1], [day day]);
                h.LineStyle = ':';
                h.Color     = [1 0 0];
            end
            
            % set coordinates for y axis
            ydaycoords = ycoords(ndays, day, firstsat,weekday);
            
            % runs if the phone use ends on the next day:
            if time(x) >= day
                % ...draws until midnight
                a = patch([begin 1 1 begin], ydaycoords, [0 0 0]);
                
                % ...draws 24 hours for next day if remaining duration is
                % >24hrs, until there is <24hrs less of the phone use
                while duration-(midnight-begin) > 1
                    day       = day+1;
                    
                    weekday   = weekday+1;
                    if weekday == 8
                        weekday = 1;
                    end
                    
                    duration  = duration-1;
                    ydaycoords = ycoords(ndays, day, firstsat, weekday);
                    
                    af = patch([0 1 1 0], ydaycoords, [0 0 0]);
                    [~, ~, af.FaceColor, ~, ~, af.EdgeColor] = barcodecolours(screendata, x, day, firstsat);
                end
                
                day       = day+1;
                weekday   = weekday+1;
                if weekday == 8
                    weekday = 1;
                end
                
                % ...continues drawing use from midnight the next day
                ydaycoords = ycoords(ndays, day, firstsat, weekday);
                ao = patch([0 duration-(midnight-begin) duration-(midnight-begin) 0], ydaycoords, [0 0 0]);
                [a.FaceColor, ao.FaceColor, ~, a.EdgeColor, ao.EdgeColor, ~] = barcodecolours(screendata, x, day, firstsat);
            else
                
                % draws phone use not crossing over onto next day
                a = patch([begin begin+duration begin+duration begin], ydaycoords, [0 0 0]);
                [a.FaceColor, ~, ~, a.EdgeColor, ~, ~] = barcodecolours(screendata, x, day, firstsat);
            end
            
            % gets next phone use
            x = x + 1;
        end
        
        day     = day+1;
        weekday = weekday+1;
        if weekday == 8
            weekday = 1;
        end
        
        
        % break when finished drawing all timestamps for current user
        if day > max(time) || (exist('axisy', 'var')==1 && axisy==day)
            break
        end
        
    end
    
    % format figure
    ylabel('day');
    xlabel('time');
    
    ax            = gca;
    ax.XTick      = [0, 0.25, 0.5, 0.75, 1];
    ax.XTickLabel = {'00:00', '06:00', '12:00', '18:00', '24:00'};
    ax.FontSize   = 14;
    
    % format figure based on specifications
    switch ndays
        case 0
            if strcmp('yes', alldays)
                axis([0 1 0 day-1])
            else
                axis([0 1 0 axisy])
            end
            
        case 1
            axis([0 1 0 1])
            alpha(.1);
            ax.YTick      = .5;
            ax.YTickLabel = {''};
            
        case 2
            axis([0 1 0 2])
            ax.YTick      = [.5, 1.5];
            ax.YTickLabel = {'weekend', 'weekday'};
            
            if addtitle==1
                title(unique_devices(pnum(eachID)));
            end
            
            if mod((day-firstsat),7)==0 || mod((day-firstsun),7)==0
                alpha(.25);
            else
                alpha(.1);
            end
            
        case 7
            axis([0 1 0 7])
            alpha(.2);
            ax.YTick      = 0.5:1:7.5;
            ax.YTickLabel = {'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday'};
    end
    
    if addtitle==1
        figtitle = unique_devices(pnum(eachID));
        title(figtitle);
    end
    
    % save if required
    if savefigure==1
        
        switch ndays
            case 0
        figtype = 'alldays_';
            case 1
        figtype = 'oneday_';
            case 2
        figtype = 'twodays_';
            case 7
        figtype = 'sevendays_';
        end
        
        figtitle = [figtype char(unique_devices(pnum(eachID)))];
        print(figtitle, '-dpng');
    end
    
    hold off
    
end

function ydaycoords = ycoords(ndays, day, firstsat, weekday)
% sets y-coordinates for the current phone use depending on required figure
switch ndays
    case 0
        ydaycoords = [day-1 day-1 day day];
    case 1
        ydaycoords = [0 0 1 1];
    case 2
        if mod((day-firstsat),7)==1 || mod((day-firstsat),7)==2
            ydaycoords = [0 0 1 1];
        else
            ydaycoords = [1 1 2 2];
        end
    case 7
        ydaycoords = [weekday-1 weekday-1 weekday weekday];
end

function [aFC, aoFC, afFC, aEC, aoEC, afEC] = barcodecolours(screendata, x, day, firstsat)
% reads whether the screen is on or off; if on, colour sections black, if
% off, colour section white. Include no edges. works within barcode scripts 
% (one day, 2 days, 7 days, all days).

wkend = [0 0 0];
wk = [0 0 0];

if screendata(x-1, 3)==1
    
    if mod((day-firstsat),7) == 1 || mod((day-firstsat),7) == 2
        aFC  = wkend; aEC = 'none';
        afFC = wkend; afEC = 'none';
        aoFC = wkend; aoEC = 'none';
    else
        aFC  = wk; aEC = 'none';
        afFC = wk; afEC = 'none';
        aoFC = wk; aoEC = 'none';
    end
    
else
    aFC  = 'none';  aEC = 'none';
    afFC = 'none'; afEC = 'none';
    aoFC = 'none'; aoEC = 'none';
end