function [data, unique_devices, unique_ids] = csv2data
% reads the ScreenProbe.csv file produced by funf and converts it to
% numerical data. Also includes the output of unique devices (how many and 
% strings), and unique ids (how many and strings - without the '_n' suffix). 

%% read data
clear
clc

data = open('ScreenProbe.csv');
data.textdata = data.textdata(2:end, :);

%% code user numbers into numerical data (with timestamp)

unique_devices  = unique(data.textdata(:,2), 'stable'); % creates a string with each unique device (in order)
unique_ids      = unique(data.textdata(:,1), 'stable'); % creates a string with each unique id (in order)

for n = 1:length(unique_ids)
    unique_ids(n) = {unique_ids{n}(1:36)}; % takes the number suffix off different ids
end

    new_unique_ids      = unique(unique_ids, 'stable'); % creates a string with each unique id (in order)

for n = 1:length(unique_devices)
    device = strcmp(data.textdata(:, 2),unique_devices(n));
    data.data(device, 2) = n;
    
end

for n = 1:length(new_unique_ids)
id = strcmp(data.textdata(:,1), new_unique_ids(n));
data.data(id, 3) = n;

end
%% code 0 if screen off, 1 if screen on
       
for n = 1:length(data.textdata)
    
    if strcmp(data.textdata(n, 4),'True')==1
        data.data(n, 3) = 1;
    elseif strcmp(data.textdata(n, 4), 'False')==1
        data.data(n, 3) = 0;
    end

end

data = data.data;

%% sort data by participant number and timestamp, and add durations
data = sortrows(data, [2 1]);

for row = 2:length(data)
    duration = (data(row, 1) - data(row-1, 1));

    if duration > 0
        data(row-1, 4) = duration;
    else
        data(row-1, 4) = NaN;
    end
    
end

% add timestamp in excel format
data(:, 5) = (data(:,1)/86400)+25569;

% writes data to .csv file, and device ids to .txt file
dlmwrite('SCREENSTATE_DATA.csv', data, 'delimiter', ',', 'precision',20);
dlmwrite('SCREENSTATE_DEVICES.txt', unique_devices, 'delimiter', '');

clc
