function descriptives
% DESCRIPTIVES
% get descriptive statistics from screenstate data
% must run CSV2DATA before using this script
% (folder must contain 'SCREENSTATE_DATA.csv' and 'SCREENSTATE_DEVICES.txt'

%%IMPORT DATA FROM CSV
try
    data = csvread('SCREENSTATE_DATA.csv');
    data(:, 6) = mod(data(:,5),1);
    
    fileID = fopen('SCREENSTATE_DEVICES.txt');
    ud = textscan(fileID, '%s');
    
    for n = 1 : length(ud{1,1})
        ud1 = ud{1,1}{n,1};
        unique_devices{n} = ud1;
    end
    
catch
    errordlg({'Could not find data'...
        ''...
        'Please run CSV2DATA, and ensure that you have:'...
        '     SCREENSTATE_DATA.csv & '...
        '     SCREENSTATE_DEVICES.txt'...
        'in your current directory.'})
end

clear fileID ud ud1 n
table_array = [];

% whether summary data (i.e. 1 value per p) or detailed (i.e. 1 values per day per p)
ok = 0;
while ok == 0
    [type,ok] = listdlg('PromptString','what descriptives do you want?',...
        'SelectionMode','single',...
        'ListSize', [250 50],...
        'ListString',{'one row per participant', 'one row per day, per participant', 'one row per time period, per participant'});
end

% which participant number you want to do (if only one)
ok = 0;
while ok == 0
    [pnum,ok] = listdlg('PromptString','which participants do you want?',...
        'SelectionMode','multiple',...
        'ListSize',[250 250],...
        'ListString',unique_devices);
end

% histogram parameters dialog box
% when you want to cover from (in seconds)
% when you want to cover to (in seconds)
% how big you want the bins to be for histograms (in seconds)
ok = 0;
while ok == 0
    [histtype,ok] = listdlg('PromptString','Do you want standard or log-transformed frequency data?',...
        'SelectionMode','single',...
        'ListSize',[250 40],...
        'ListString',{'standard', 'log-transformed'});
end

if histtype == 1
    answer = inputdlg({'where do you want to start? (seconds)',...
        'where do you want to end? (seconds)',...
        'how big do you want the bins to be? (seconds)'},...
        'hist params', 1, {'0', '15', '1'});
    
    seconds_from = str2double(answer{1});
    seconds_to   = str2double(answer{2});
    bin_size     = str2double(answer{3});
    
    edges = seconds_from : bin_size : seconds_to;
    
elseif histtype == 2
    
    edges = 10.^(0:1:6); edges = [0 edges]; hdur = [];
    
end

for n = 1: length(edges)-1
    countvars{n}    = ['count_', int2str(edges(n)), 'to', int2str(edges(n+1))];
    totvars{n}      = ['tot_', int2str(edges(n)), 'to', int2str(edges(n+1))];
    percvars{n}     = ['perc_', int2str(edges(n)), 'to', int2str(edges(n+1))];
end

% whether you want it just to do 'on' durations
ok = 0;
while ok == 0
    [onoff, ok] = listdlg('PromptString','do you want data from on and / or off durations?',...
        'SelectionMode','multiple',...
        'ListSize',[250 40],...
        'InitialValue', 2,...
        'ListString',{'off', 'on'});
    onoff = onoff - 1;
end

clear ok answer n

%% get set variable names / create files, and get descriptives
    % for debugging:
    %   sState = 1; p_num = 1;

for sState = onoff(1) : onoff(end)
    for p_num = 1 : length(pnum)
        p = pnum(p_num);
        today = 0;
        % get current participant timestamps & set variable names / create files
        thisp = (data(:, 2)==p);
        
        % generate filename
        if sState == 0
            statetext = '_off';
        elseif sState == 1
            statetext = '_on';
        end
        
        if type==1 % type 1 = one overall value for each p
            values_per_participant = 1; firstcol = 'user';
            filename = ['alldays' statetext '.csv'];
            
        elseif  type==2 % type 2 = one value for each day
            values_per_participant = (ceil(max(data(thisp, 5))) - floor(min(data(thisp, 5))));
            firstcol = 'day';
            filename = strcat(['eachday' statetext '_'], unique_devices(:,p), '.csv'); filename = filename{:};
            
        elseif  type==3 % type 3 = one value for each time of day
            values_per_participant = 4 * (ceil(max(data(thisp, 5))) - floor(min(data(thisp, 5))));
            firstcol = 'time_of_day';
            filename = strcat(['eachtime' statetext '_'], unique_devices(:,p), '.csv'); filename = filename{:};
            
        end
        
        % get the appropriate data, and calculate descriptives
        for day = 1 : values_per_participant
            
            if type==1
                rows = (data(:,3)==sState) & (data(:, 2)==p);
                data_rows = data(rows,:);
                
                for n = 1:length(pnum)
                    rowvalues{n} = num2str(pnum(n));
                end
                
            elseif type==2
                rows = (data(:,3)==sState) & (data(:, 2)==p) & ...
                    data(:,5)<=((ceil(min(data(thisp,5))))+(day-1)) & ...
                    data(:,5)>=((floor(min(data(thisp,5))))+(day-1));
                data_rows = data(rows,:);
                
                for n = 1:values_per_participant
                    rowvalues{n} = num2str(n);
                end
                
            elseif type==3
                if mod(day,4)==0
                    endtime = 1;
                else
                    endtime = mod(day,4) * .25;
                end
                begintime = endtime - .25;
                
                if mod(day,4)==1
                    today = today + 1;
                end
                rows = (data(:,3)==sState) & (data(:, 2)==p) & ...
                    data(:,5)<=((ceil(min(data(thisp,5))))+(today-1)) & ...
                    data(:,5)>=((floor(min(data(thisp,5))))+(today-1)) & ...
                    mod(data(:,6),1) >= begintime & ...
                    mod(data(:,6),1) < endtime;
                data_rows = data(rows,:);
                    d = 1;
                    for n = 1:(values_per_participant/4)
                        
                        rowvalues{d} =     ['Night (00:00-06:00) - day ', num2str(n)];
                        rowvalues{d+1} =   ['Morning (06:00-12:00) - day ', num2str(n)];
                        rowvalues{d+2} = ['Afternoon (12:00-18:00) - day ', num2str(n)];
                        rowvalues{d+3} =   ['Evening (18:00-24:00) - day ', num2str(n)];
                        d = d + 4;
                        
                    end % end: for n = 1:values_per_participant
                                    
            end % end: if type==0
            
            % get descriptives
            nD = (ceil(max(data(thisp, 5))) - floor(min(data(thisp, 5))));
            
            meanD       = nanmean(data_rows(:, 4));
            medianD     = nanmedian(data_rows(:, 4));
            [md, mfd]   = mode(data_rows(:, 4));
            modeD       = md;
            modeFreqD	= mfd ;
            stdD        = nanstd(data_rows(:, 4));
            
            minD        = min(data_rows(:, 4));
            if isempty(minD)==1
                minD = NaN;
            end
            
            maxD        = max(data_rows(:, 4));
            if isempty(maxD)==1
                maxD = NaN;
            end
            
            
            sumD        = nansum(data_rows(:, 4));
            sumhrsD     = (sumD)/3600;
            
            hcount  = histc((data_rows(:, 4)), edges); hcount  = hcount(1:end-1);
            hperc   = (hcount/(sum(hcount))) * 100; % to make perc / freq
            hdur = [];
            for n = 1:length(edges)-1
                sh = (data_rows(:,4)>=edges(n)) & (data_rows(:,4)<edges(n+1));
                hdur = [hdur sum(data_rows(sh,4))];
            end
            
            if type == 2 || type == 3
                try
                    table_array = [table_array; day nD sumD sumhrsD meanD stdD medianD...
                        modeD modeFreqD minD maxD hcount' hperc' hdur];
                    clear nD sumD sumhrsD meanD stdD medianD modeD modeFreqD minD maxD hcount' hperc' hdur;
                catch
                    table_array = [table_array; day nD NaN(1,(length(hcount)*3 + 9))];
                    clear nD sumD sumhrsD meanD stdD medianD modeD modeFreqD minD maxD hcount' hperc' hdur;
                end
            end
        end
        
        if type == 1
            try
                table_array = [table_array; p nD sumD sumhrsD meanD stdD medianD...
                    modeD modeFreqD minD maxD hcount' hperc' hdur];
                clear nD sumD sumhrsD meanD stdD medianD modeD modeFreqD minD maxD hcount' hperc' hdur;
            catch
                table_array = [table_array; p nD NaN(1,(length(hcount)*3 + 9))];
                clear nD sumD sumhrsD meanD stdD medianD modeD modeFreqD minD maxD hcount' hperc' hdur;
            end
        end
        if type == 2 || type == 3
            variablevalues = [firstcol 'num_days' 'duration' 'duration_hrs'...
                'mean' 'std' 'median' 'mode' 'mode_freq' 'min' 'max'...
                countvars, percvars, totvars];
            try
                T = array2table(table_array,...
                    'RowNames', rowvalues,...
                    'VariableNames', variablevalues);
                writetable(T, filename,'WriteRowNames',true);
                
            catch
                err = ['could not write table for ', unique_devices(:,p)];
                dlmwrite('error.txt',err,'delimiter','','-append');
            end
            table_array = [];
            clear nD sumD sumhrsD meanD stdD medianD modeD modeFreqD minD maxD hcount' hperc' hdur rowvalues;
        end
    end
    
    if type == 1
        variablevalues = [firstcol 'num_days' 'duration' 'duration_hrs'...
            'mean' 'std' 'median' 'mode' 'mode_freq' 'min' 'max'...
            countvars, percvars, totvars];
        try
            T = array2table(table_array,...
                'RowNames', rowvalues,...
                'VariableNames', variablevalues);
            writetable(T, filename,'WriteRowNames',true);
            
        catch
            err = 'could not write table';
            dlmwrite('error.txt',err,'delimiter','','-append');
        end
        table_array = [];
        clear nD sumD sumhrsD meanD stdD medianD modeD modeFreqD minD maxD hcount' hperc' hdur rowvalues;
    end
end

function y = nanmean(x)
if isempty(x)
	y = NaN;
	return
end

if nargin < 2
	dim = min(find(size(x)~=1));
	if isempty(dim)
		dim = 1;
	end
end

% Replace NaNs with zeros.
nans = isnan(x);
x(isnan(x)) = 0; 

% denominator
count = size(x,dim) - sum(nans,dim);

% Protect against a  all NaNs in one dimension
i = find(count==0);
count(i) = ones(size(i));

y = sum(x,dim)./count;
y(i) = i + NaN;

function y = nanmedian(x)
if isempty(x)
	y = [];
	return
end

if nargin < 2
	dim = min(find(size(x)~=1));
	if isempty(dim)
		dim = 1;
	end
end

siz  = size(x);
n    = size(x,dim);

% Permute and reshape so that DIM becomes the row dimension of a 2-D array
perm = [dim:max(length(size(x)),dim) 1:dim-1];
x = reshape(permute(x,perm),n,prod(siz)/n);

% force NaNs to bottom of each column
x = sort(x,1);

% identify and replace NaNs
nans = isnan(x);
x(isnan(x)) = 0;

% number of non-NaN element in each column
s = size(x,1) - sum(nans);
y = zeros(size(s));

% now calculate median for every element in y
% (does anybody know a more eefficient way than with a 'for'-loop?)
for i = 1:length(s)
	if rem(s(i),2) && s(i) > 0
		y(i) = x((s(i)+1)/2,i);
	elseif rem(s(i),2)==0 && s(i) > 0
		y(i) = (x(s(i)/2,i) + x((s(i)/2)+1,i))/2;
	end
end

% Protect against a column of NaNs
i = find(y==0);
y(i) = i + nan;

% permute and reshape back
siz(dim) = 1;
y = ipermute(reshape(y,siz(perm)),perm);

function y = nanstd(x)
if isempty(x)
	y = NaN;
	return
end

if nargin < 3
	flag = 0;
end

if nargin < 2
	dim = min(find(size(x)~=1));
	if isempty(dim)
		dim = 1; 
	end	  
end


% Find NaNs in x and nanmean(x)
nans = isnan(x);
avg = nanmean(x);

% create array indicating number of element 
% of x in dimension DIM (needed for subtraction of mean)
tile = ones(1,max(ndims(x),dim));
tile(dim) = size(x,dim);

% remove mean
x = x - repmat(avg,tile);

count = size(x,dim) - sum(nans,dim);

% Replace NaNs with zeros.
x(isnan(x)) = 0; 


% Protect against a  all NaNs in one dimension
i = find(count==0);

if flag == 0
	y = sqrt(sum(x.*x,dim)./max(count-1,1));
else
	y = sqrt(sum(x.*x,dim)./max(count,1));
end
y(i) = i + NaN;

function y = nansum(x, dim)

if isempty(x)
	y = [];
	return
end

if nargin < 2
	dim = min(find(size(x)~=1));
	if isempty(dim)
		dim = 1;
	end
end

% Replace NaNs with zeros.
nans = isnan(x);
x(isnan(x)) = 0; 

% Protect against all NaNs in one dimension
count = size(x,dim) - sum(nans,dim);
i = find(count==0);

y = sum(x,dim);
y(i) = NaN;



